@@ -1,5 +1,3 @@
-from __future__ import unicode_literals
-
 import logging
 import os
 import sys
@@ -24,10 +22,10 @@ if lib_path:
     lib_names = None
 elif os.name == 'nt':
     # Windows NT shared libraries
-    lib_names = [str('gdal201'), str('gdal20'), str('gdal111'), str('gdal110'), str('gdal19')]
+    lib_names = ['gdal202', 'gdal201', 'gdal20', 'gdal111', 'gdal110', 'gdal19']
 elif os.name == 'posix':
     # *NIX library names.
-    lib_names = ['gdal', 'GDAL', 'gdal2.1.0', 'gdal2.0.0', 'gdal1.11.0', 'gdal1.10.0', 'gdal1.9.0']
+    lib_names = ['gdal', 'GDAL', 'gdal2.2.0', 'gdal2.1.0', 'gdal2.0.0', 'gdal1.11.0', 'gdal1.10.0', 'gdal1.9.0']
 else:
     raise ImproperlyConfigured('GDAL is unsupported on OS "%s".' % os.name)
 
@@ -60,7 +58,7 @@ if os.name == 'nt':
 
 def std_call(func):
     """
-    Returns the correct STDCALL function for certain OSR routines on Win32
+    Return the correct STDCALL function for certain OSR routines on Win32
     platforms.
     """
     if os.name == 'nt':
@@ -71,19 +69,19 @@ def std_call(func):
 
 # #### Version-information functions. ####
 
-# Returns GDAL library version information with the given key.
+# Return GDAL library version information with the given key.
 _version_info = std_call('GDALVersionInfo')
 _version_info.argtypes = [c_char_p]
 _version_info.restype = c_char_p
 
 
 def gdal_version():
-    "Returns only the GDAL version number information."
+    "Return only the GDAL version number information."
     return _version_info(b'RELEASE_NAME')
 
 
 def gdal_full_version():
-    "Returns the full GDAL version information."
+    "Return the full GDAL version information."
     return _version_info('')
 
 
