@@ -1,5 +1,5 @@
 # Autoreloading launcher.
-# Borrowed from Peter Hunt and the CherryPy project (http://www.cherrypy.org).
+# Borrowed from Peter Hunt and the CherryPy project (https://cherrypy.org/).
 # Some taken from Ian Bicking's Paste (http://pythonpaste.org/).
 #
 # Portions copyright (c) 2004, CherryPy Team (team@cherrypy.org)
@@ -42,7 +42,7 @@ from django.conf import settings
 from django.core.signals import request_finished
 
 # This import does nothing, but it's necessary to avoid some race conditions
-# in the threading module. See http://code.djangoproject.com/ticket/2330 .
+# in the threading module. See https://code.djangoproject.com/ticket/2330 .
 try:
     import threading  # NOQA
 except ImportError:
@@ -280,10 +280,16 @@ def reloader_thread():
 
 
 def restart_with_reloader():
+    import django.__main__
     while True:
-        args = [sys.executable] + ['-W%s' % o for o in sys.warnoptions] + sys.argv
-        new_environ = os.environ.copy()
-        new_environ["RUN_MAIN"] = 'true'
+        args = [sys.executable] + ['-W%s' % o for o in sys.warnoptions]
+        if sys.argv[0] == django.__main__.__file__:
+            # The server was started with `python -m django runserver`.
+            args += ['-m', 'django']
+            args += sys.argv[1:]
+        else:
+            args += sys.argv
+        new_environ = {**os.environ, 'RUN_MAIN': 'true'}
         exit_code = subprocess.call(args, env=new_environ)
         if exit_code != 3:
             return exit_code
@@ -307,24 +313,11 @@ def python_reloader(main_func, args, kwargs):
             pass
 
 
-def jython_reloader(main_func, args, kwargs):
-    from _systemrestart import SystemRestart
-    _thread.start_new_thread(main_func, args)
-    while True:
-        if code_changed():
-            raise SystemRestart
-        time.sleep(1)
-
-
 def main(main_func, args=None, kwargs=None):
     if args is None:
         args = ()
     if kwargs is None:
         kwargs = {}
-    if sys.platform.startswith('java'):
-        reloader = jython_reloader
-    else:
-        reloader = python_reloader
 
     wrapped_main_func = check_errors(main_func)
-    reloader(wrapped_main_func, args, kwargs)
+    python_reloader(wrapped_main_func, args, kwargs)
